<?php

if (!defined('ABSPATH')) exit;

class d4ppn_WalkerPageFlat extends Walker_Page {
    function start_lvl(&$output, $depth) {
        $output.= "";
    }

    function end_lvl(&$output, $depth) {
        $output.= "";
    }
}

class d4ppn_gdPagesNavigator extends gdr2_Widget {
    var $widget_base = "d4p_pages_navigator";
    var $widget_domain = "d4p_pages_navigator";
    var $cache_prefix = "d4ppn";

    var $title = "";

    var $defaults = array(
        "title" => "Pages",
        "_display" => "all",
        "auto_title" => 1,
        "post_type" => "page",
        "method" => "full",
        "hierarchy" => 1,
        "level_up" => 1,
        "parent" => 0,
        "depth" => 0,
        "sort" => "menu_order",
        "order" => "asc",
        "exclude" => array()
    );

    function __construct($id_base = false, $name = "", $widget_options = array(), $control_options = array()) {
        $this->widget_description = __("Enhanced pages navigation.", "gd-pages-navigator");
        $this->widget_name = "GD Pages Navigator";
        parent::__construct($this->widget_base, $this->widget_name, array(), array('width' => 500));
    }

    function form($instance) {
        global $gdpn_core;
        $instance = wp_parse_args((array)$instance, $this->defaults);

        echo '<div class="d4p-gdpn-widget">';
        include(GDPAGESNAVIGATOR_PATH.'widget/form.php');
        echo '</div>';
    }

    function title($instance) {
        if ($this->title == "") {
            return $instance["title"];
        } else {
            return $this->title;
        }
    }

    function get_method() {
        return array(
            "full" => __("Full List", "gd-pages-navigator"),
            "parent" => __("Custom Parent", "gd-pages-navigator"),
            "navigate" => __("Navigate", "gd-pages-navigator")
        );
    }

    function get_order() {
        return array(
            "asc" => __("Ascending", "gd-pages-navigator"),
            "desc" => __("Descending", "gd-pages-navigator")
        );
    }

    function get_sort() {
        return array(
            "menu_order" => __("Menu Order", "gd-pages-navigator"),
            "post_title" => __("Page Title", "gd-pages-navigator"),
            "post_name" => __("Page Slug", "gd-pages-navigator"),
            "ID" => __("Page ID", "gd-pages-navigator"),
            "post_date" => __("Cration Date", "gd-pages-navigator"),
            "post_modified" => __("Modification Date", "gd-pages-navigator")
        );
    }

    function update($new_instance, $old_instance) {
        $instance = $old_instance;

        $instance["title"] = strip_tags(stripslashes($new_instance["title"]));
        $instance["_display"] = strip_tags(stripslashes($new_instance["_display"]));
        $instance["auto_title"] = isset($new_instance["auto_title"]) ? 1 : 0;
        $instance["hierarchy"] = isset($new_instance["hierarchy"]) ? 1 : 0;
        $instance["level_up"] = isset($new_instance["level_up"]) ? 1 : 0;
        $instance["depth"] = intval(stripslashes($new_instance["depth"]));
        $instance["post_type"] = strip_tags(stripslashes($new_instance["post_type"]));
        $instance["method"] = strip_tags(stripslashes($new_instance["method"]));
        $instance["sort"] = strip_tags(stripslashes($new_instance["sort"]));
        $instance["order"] = strip_tags(stripslashes($new_instance["order"]));
        $instance["parent"] = strip_tags(stripslashes($new_instance["parent"]));

        $instance["exclude"] = strip_tags(stripslashes($new_instance["exclude"]));
        $instance["exclude"] = explode(",", $instance["exclude"]);
        $instance["exclude"] = array_map("trim", $instance["exclude"]);
        $instance["exclude"] = array_map("intval", $instance["exclude"]);
        $instance["exclude"] = array_diff($instance["exclude"], array(0));
        $instance["exclude"] = array_unique($instance["exclude"]);

        return $instance;
    }

    function list_full($instance, $args) {
        return "<ul>".wp_list_pages($args)."</ul>";
    }

    function list_parent($instance, $args) {
        $args["child_of"] = $instance["parent"];
        $page = get_page($instance["parent"]);
        if ($instance["auto_title"]) {
            $this->title = $page->post_title;
        }
        return "<ul>".wp_list_pages($args)."</ul>";
    }

    function list_navigate($instance, $args) {
        if (is_singular($args["post_type"])) {
            global $post;

            if ($this->has_children($post->ID)) {
                $args["child_of"] = $post->ID;
            } else {
                $args["child_of"] = $post->post_parent;
            }

            if ($instance["level_up"] == 1 && $post->post_parent > 0) {
                $parent = get_page($post->post_parent);
                $args["title_li"] = '<a href="'.get_permalink($post->post_parent).'">'.$parent->post_title.'</a>';
            }

            if ($instance["auto_title"] == 1) {
                $this->title = $post->post_title;
            }
        }

        return "<ul>".wp_list_pages($args)."</ul>";
    }

    function has_children($post_id) {
        $children = get_pages("child_of=".$post_id);
        if (count($children) > 0) {
            return $children;
        } else {
            return false;
        }
    }

    function results($instance) {
        $args = array("title_li" => "", "echo" => 0);
        if ($instance["hierarchy"] == 0) {
            $args["walker"] = new d4ppn_WalkerPageFlat();
        }
        $args["depth"] = $instance["depth"];
        $args["post_type"] = $instance["post_type"];
        $args["sort_order"] = $instance["order"];
        $args["sort_column"] = $instance["sort"];
        if ($args["sort_column"] == "menu_order") {
            $args["sort_column"] = "menu_order, post_title";
        }

        $results = "";
        switch ($instance["method"]) {
            default:
            case "full":
                $results = $this->list_full($instance, $args);
                break;
            case "parent":
                $results = $this->list_parent($instance, $args);
                break;
            case "navigate":
                $results = $this->list_navigate($instance, $args);
                break;
        }
        return $results;
    }

    function render($results, $instance) {
        $render = '<div class="d4p-widget d4p-pages_navigator">'.GDR2_EOL;
        $render.= $results;
        $render.= '</div>'.GDR2_EOL;
        return $render;
    }
}

?>